@@ -25,6 +25,7 @@
 
 #include "eog-thumb-view.h"
 #include "eog-list-store.h"
+#include "eog-pixbuf-cell-renderer.h"
 #include "eog-image.h"
 
 #ifdef HAVE_EXIF
@@ -55,6 +56,7 @@ struct _EogThumbViewPrivate {
 	gint start_thumb; /* the first visible thumbnail */
 	gint end_thumb;   /* the last visible thumbnail  */
 	GtkWidget *menu;  /* a contextual menu for thumbnails */
+	GtkCellRenderer *pixbuf_cell;
 };
 
 /* Drag 'n Drop */
@@ -193,7 +195,7 @@ tb_on_visible_range_changed_cb (EogThumbView *tb,
 	
 	eog_thumb_view_update_visible_range (tb, gtk_tree_path_get_indices (path1) [0],
 					     gtk_tree_path_get_indices (path2) [0]);
-	
+
 	gtk_tree_path_free (path1);
 	gtk_tree_path_free (path2);
 }
@@ -295,6 +297,8 @@ tb_on_button_press_event_cb (GtkWidget *tb, GdkEventButton *event,
 
 	    eog_thumb_view_popup_menu (EOG_THUMB_VIEW (tb), event);
 	    
+	    gtk_tree_path_free (path);
+
 	    return TRUE;
     }
     
@@ -365,18 +369,15 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 	ExifData *exif_data;
 #endif	
 
-	if (keyboard_mode) {
-		image = eog_thumb_view_get_first_selected_image (EOG_THUMB_VIEW (widget));
-	} else {
-		if (!gtk_icon_view_get_item_at_pos (GTK_ICON_VIEW (widget),
-						    x, y, &path, NULL)) {
-			return FALSE;
-		}
-
-		image = eog_thumb_view_get_image_from_path (EOG_THUMB_VIEW (widget), path);
-
-		gtk_tree_path_free (path);
+	if (!gtk_icon_view_get_tooltip_context (GTK_ICON_VIEW (widget), 
+						&x, &y, keyboard_mode,
+						NULL, &path, NULL)) {
+		return FALSE;
 	}
+	
+	image = eog_thumb_view_get_image_from_path (EOG_THUMB_VIEW (widget),
+						    path);
+	gtk_tree_path_free (path);
 
 	if (image == NULL) {
 		return FALSE;
@@ -419,7 +420,7 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 					  eog_image_get_caption (image),
 					  width, 
 					  height, 
-					  ngettext ("pixels", "pixels", height), 
+					  ngettext ("pixel", "pixels", height), 
 					  bytes,
 					  type_str);
 
@@ -457,17 +458,34 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 static void
 eog_thumb_view_init (EogThumbView *tb)
 {
+	tb->priv = EOG_THUMB_VIEW_GET_PRIVATE (tb);
+	
+	tb->priv->pixbuf_cell = eog_pixbuf_cell_renderer_new ();
+
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (tb), 
+	      	  		    tb->priv->pixbuf_cell, 
+	      			    FALSE);
+	
+	g_object_set (tb->priv->pixbuf_cell, 
+	              "follow-state", TRUE, 
+	              "height", 100, 
+	              "yalign", 0.5, 
+	              "xalign", 0.5, 
+	              NULL);
+	
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (tb),
+	                                tb->priv->pixbuf_cell, 
+	      		                "pixbuf", EOG_LIST_STORE_THUMBNAIL,
+	                                NULL);
+
 	gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (tb),
  					  GTK_SELECTION_MULTIPLE);
-
-	gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (tb),
-					 EOG_LIST_STORE_THUMBNAIL);
 	
 	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (tb),
 					  EOG_THUMB_VIEW_SPACING);
 
 	gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (tb),
-					  EOG_THUMB_VIEW_SPACING);
+				       EOG_THUMB_VIEW_SPACING);
 
 #ifdef HAVE_GTK_TOOLTIP
 	g_object_set (tb, "has-tooltip", TRUE, NULL);
@@ -478,8 +496,6 @@ eog_thumb_view_init (EogThumbView *tb)
 			  NULL);
 #endif /* HAVE_GTK_TOOLTIP */	
 	
-	tb->priv = EOG_THUMB_VIEW_GET_PRIVATE (tb);
-	
 	tb->priv->start_thumb = 0;
 	tb->priv->end_thumb = 0;
 	tb->priv->menu = NULL;
@@ -523,6 +539,16 @@ eog_thumb_view_set_model (EogThumbView *tb, EogListStore *store)
 	}
 }
 
+void
+eog_thumb_view_set_item_height (EogThumbView *tb, gint height)
+{
+	g_return_if_fail (EOG_IS_THUMB_VIEW (tb));
+
+	g_object_set (tb->priv->pixbuf_cell, 
+	              "height", height, 
+	              NULL);
+}
+
 static void
 eog_thumb_view_get_n_selected_helper (GtkIconView *tb,
 				      GtkTreePath *path,
